checked-exceptions:plugin
Safe HaskellNone
LanguageGHC2021

Control.Monad.CheckedExcept.Plugin

Description

Type checking plugin to assist with unification of weakened exceptions.

Synopsis

Documentation

plugin :: Plugin Source #

Help resolve ambiguous exception-set metavariables in QualifiedDo.>>=.

Uses defaultingPlugin only (tcPlugin is disabled; the old rewrite/solve path is removed — code that relied on fiat coercions must use defaulting).